==> data_structures-1.0.0.tar.gz: data_structures/src/DataStructure/Set.php <==
<?php

declare(strict_types = 1);

namespace Drupal\data_structures\DataStructure;

/**
 * A data structure for sets.
 *
 * By definition, every element of a set is unique within the set.
 */
class Set implements \IteratorAggregate, \JsonSerializable, \Countable {

  /**
   * Internal storage for set members.
   *
   * @var array
   */
  protected array $members = [];

  /**
   * Optionally add values at Set construction.
   *
   * @param iterable|null $values
   *   Contains a variable number of values to add to the set.
   */
  public function __construct(?iterable $values) {
    if (!is_null($values)) {
      $this->add(...$values);
    }
  }

  /**
   * Returns set data as an array.
   *
   * @return array
   *   The set values.
   */
  public function toArray(): array {
    return $this->members;
  }

  /* ***** Interface Methods **** */

  /**
   * {@inheritdoc}
   */
  public function getIterator(): \Traversable {
    return new \ArrayIterator($this->members);
  }

  /**
   * {@inheritdoc}
   */
  public function count(): int {
    return count($this->members);
  }

  /**
   * {@inheritdoc}
   */
  public function jsonSerialize(): mixed {
    return $this->members;
  }

  /* ***** Maintaining the Set **** */

  /**
   * Empties the set.
   */
  public function empty(): void {
    $this->members = [];
  }

  /**
   * Determines if the set is empty.
   *
   * @return bool
   *   True if the set is empty.
   */
  public function isEmpty(): bool {
    return empty($this->members);
  }

  /**
   * Adds new values to the set if they are not already members.
   *
   * @param mixed ...$values
   *   A variable number of values to add to the set.
   */
  public function add(...$values): void {
    foreach ($values as $value) {
      if (!$this->has($value)) {
        $this->members[] = $value;
      }
    }
  }

  /**
   * Removes values from the set if they are members.
   *
   * @param mixed ...$values
   *   A variable number of values to remove from the set.
   */
  public function remove(mixed ...$values): void {
    $filterSet = new Set(NULL);
    $filterSet->add(...$values);
    // $filter finds values not in the filter set.
    $filter = fn($value) => !$filterSet->has($value);
    $filtered = array_filter($this->members, $filter);
    $this->members = $filtered;
  }

  /**
   * Does the set have the value as a member?.
   *
   * @param mixed $value
   *   The value to check.
   *
   * @return bool
   *   True if the value is in the set
   */
  public function has(mixed $value):bool {
    $found = FALSE;
    foreach ($this->members as $member) {
      if ($this->membersAreEqual($member, $value)) {
        $found = TRUE;
        break;
      }
    }
    return $found;
  }

  /**
   * Finds the union with another set.
   *
   * +----------+
   * |$this.....|
   * |.....+-----------+
   * |.....|....|......|
   * |.....|....|......|
   * +-----|----+......|
   *       |...........|
   *       |......$set |
   *       +-----------+
   *
   * Union combines the members of both sets.
   *
   * @param \Drupal\data_structures\DataStructure\Set $set
   *   The set to use in $this ∪ $set.
   *
   * @return \Drupal\data_structures\DataStructure\Set
   *   The intersection.
   */
  public function union(Set $set): Set {
    $union = new \AppendIterator();
    $union->append(new \IteratorIterator($this));
    $union->append(new \IteratorIterator($set));
    return new Set($union);
  }

  /**
   * Finds the intersection with another set.
   *
   * +----------+
   * |$this     |
   * |     +-----------+
   * |     |....|      |
   * |     |....|      |
   * +-----|----+      |
   *       |           |
   *       |      $set |
   *       +-----------+
   *
   * Intersection finds the items that are members of both sets.
   *
   * @param \Drupal\data_structures\DataStructure\Set $set
   *   The set to use in $this ∩ $set.
   *
   * @return \Drupal\data_structures\DataStructure\Set
   *   The intersection.
   */
  public function intersect(Set $set): Set {
    $filter = fn($value) => $set->has($value);
    $intersection = array_filter($this->members, $filter);
    return new Set($intersection);
  }

  /**
   * Finds the difference with another set.
   *
   * +----------+
   * |$this.....|
   * |.....+-----------+
   * |.....|    |      |
   * |.....|    |      |
   * +-----|----+      |
   *       |           |
   *       |      $set |
   *       +-----------+
   *
   * Difference finds the items that are members of the first set but not the
   * second.
   *
   * @param \Drupal\data_structures\DataStructure\Set $set
   *   The set to use in $this \ $set.
   *
   * @return \Drupal\data_structures\DataStructure\Set
   *   The intersection.
   */
  public function difference(Set $set): Set {
    $filter = fn($value) => !$set->has($value);
    $intersection = array_filter($this->members, $filter);
    return new Set($intersection);
  }

  /**
   * Finds the symmetric difference with another set.
   *
   * +----------+
   * |$this.....|
   * |.....+-----------+
   * |.....|    |......|
   * |.....|    |......|
   * +-----|----+......|
   *       |...........|
   *       |......$set |
   *       +-----------+
   *
   * Symmetric difference finds the items that are members of both sets that
   * are not also members of both sets.
   *
   * @param \Drupal\data_structures\DataStructure\Set $set
   *   The set to use in $this \ $set.
   *
   * @return \Drupal\data_structures\DataStructure\Set
   *   The intersection.
   */
  public function symDifference(Set $set): Set {
    return $this->difference($set)->union($set->difference($this));
  }

  /**
   * Applies a callable to re-map the set.
   *
   * @param callable $callable
   *   A callable that accepts a Set member and returns a new value.
   *
   * @return \Drupal\data_structures\DataStructure\Set
   *   The result of the mapping.
   */
  public function map(callable $callable): Set {
    return new static(array_map($callable, $this->members));
  }

  /**
   * Applies a callable to reduce the set to a single value.
   *
   * @param callable $callable
   *   The callable signature is: callback(mixed $carry, mixed $item): mixed.
   * @param mixed $initial
   *   The initial value to pass as the "carry" value to the callable.
   *
   * @return mixed
   *   Returns the last return value from the callback.
   */
  public function reduce(callable $callable, mixed $initial = NULL): mixed {
    return array_reduce($this->members, $callable, $initial);
  }

  /**
   * Applies a callable to filter the set into a new set.
   *
   * @param callable $callable
   *   The callable signature is: callback(mixed $item): bool.
   *
   * @return \Drupal\data_structures\DataStructure\Set
   *   The filtered sequence..
   */
  public function filter(callable $callable): Set {
    return new static(array_filter($this->members, $callable));
  }

  /* ***** Internal Methods **** */

  /**
   * Comparison function to determine equality.
   *
   * If both members implement EqualityInterface, then they define equality
   * for themselves.  Otherwise the === operator will be used to determine
   * equality.
   *
   * @param mixed $one
   *   An existing or possible member of the set.
   * @param mixed $two
   *   An existing or possible member of the set.
   *
   * @return bool
   *   True if $one and $two are equal.
   */
  protected function membersAreEqual(mixed $one, mixed $two): bool {
    if ($one instanceof EqualityInterface && $two instanceof EqualityInterface) {
      return get_class($one) === get_class($two) && $one->equalTo($two);
    }
    return $one === $two;
  }

}

==> data_structures-1.0.1.tar.gz: data_structures/src/DataStructure/Set.php <==
<?php

declare(strict_types = 1);

namespace Drupal\data_structures\DataStructure;

/**
 * A general implementation for Set.
 */
class Set implements SetInterface {

  /**
   * Internal storage for set members.
   *
   * @var array
   */
  protected array $members = [];

  /**
   * Optionally add values at Set construction.
   *
   * @param iterable|null $values
   *   Contains a variable number of values to add to the set.
   */
  final public function __construct(?iterable $values) {
    if (!is_null($values)) {
      $this->add(...$values);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function toArray(): array {
    return $this->members;
  }

  /* ***** Interface Methods **** */

  /**
   * {@inheritdoc}
   */
  public function getIterator(): \Traversable {
    return new \ArrayIterator($this->members);
  }

  /**
   * {@inheritdoc}
   */
  public function count(): int {
    return count($this->members);
  }

  /**
   * {@inheritdoc}
   */
  public function jsonSerialize(): array {
    return $this->members;
  }

  /* ***** Maintaining the Set **** */

  /**
   * {@inheritdoc}
   */
  public function empty(): void {
    $this->members = [];
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty(): bool {
    return empty($this->members);
  }

  /**
   * {@inheritdoc}
   */
  public function add(...$values): void {
    foreach ($values as $value) {
      if (!$this->has($value)) {
        $this->members[] = $value;
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function remove(mixed ...$values): void {
    $filterSet = new Set(NULL);
    $filterSet->add(...$values);
    // $filter finds values not in the filter set.
    $filter = fn($value) => !$filterSet->has($value);
    $filtered = array_filter($this->members, $filter);
    $this->members = $filtered;
  }

  /**
   * {@inheritdoc}
   */
  public function has(mixed $value):bool {
    $found = FALSE;
    foreach ($this->members as $member) {
      if ($this->membersAreEqual($member, $value)) {
        $found = TRUE;
        break;
      }
    }
    return $found;
  }

  /**
   * {@inheritdoc}
   */
  public function union(SetInterface $set): Set {
    $union = new \AppendIterator();
    $union->append(new \IteratorIterator($this));
    $union->append(new \IteratorIterator($set));
    return new Set($union);
  }

  /**
   * {@inheritdoc}
   */
  public function intersect(SetInterface $set): Set {
    $filter = fn($value) => $set->has($value);
    $intersection = array_filter($this->members, $filter);
    return new Set($intersection);
  }

  /**
   * {@inheritdoc}
   */
  public function difference(SetInterface $set): Set {
    $filter = fn($value) => !$set->has($value);
    $intersection = array_filter($this->members, $filter);
    return new Set($intersection);
  }

  /**
   * {@inheritdoc}
   */
  public function symDifference(SetInterface $set): Set {
    return $this->difference($set)->union($set->difference($this));
  }

  /**
   * {@inheritdoc}
   */
  public function map(callable $callable): Set {
    return new static(array_map($callable, $this->members));
  }

  /**
   * {@inheritdoc}
   */
  public function reduce(callable $callable, mixed $initial = NULL): mixed {
    return array_reduce($this->members, $callable, $initial);
  }

  /**
   * {@inheritdoc}
   */
  public function filter(callable $callable): Set {
    return new static(array_filter($this->members, $callable));
  }

  /* ***** Internal Methods **** */

  /**
   * Comparison function to determine equality.
   *
   * If both members implement EqualityInterface, then they define equality
   * for themselves.  Otherwise the === operator will be used to determine
   * equality.
   *
   * @param mixed $one
   *   An existing or possible member of the set.
   * @param mixed $two
   *   An existing or possible member of the set.
   *
   * @return bool
   *   True if $one and $two are equal.
   */
  protected function membersAreEqual(mixed $one, mixed $two): bool {
    if ($one instanceof EqualityInterface && $two instanceof EqualityInterface) {
      return get_class($one) === get_class($two) && $one->equalTo($two);
    }
    return $one === $two;
  }

}
